<?php

require_once dirname(__FILE__) . '/../../includes/autoload.php';

class KredivoValidationModuleFrontController extends ModuleFrontController
{
    /**
     * @see FrontController::postProcess()
     */
    public function postProcess()
    {
        $cart = $this->context->cart;
        if ($cart->id_customer == 0 || $cart->id_address_delivery == 0 || $cart->id_address_invoice == 0 || !$this->module->active) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        // Check that this payment option is still available in case the customer changed his address just before the end of the checkout process
        $authorized = false;
        foreach (Module::getPaymentModules() as $module) {
            if ($module['name'] == 'kredivo') {
                $authorized = true;
                break;
            }
        }

        if (!$authorized) {
            die($this->module->l('This payment method is not available.', 'validation'));
        }

        $customer = new Customer($cart->id_customer);
        if (!Validate::isLoadedObject($customer)) {
            Tools::redirect('index.php?controller=order&step=1');
        }

        $currency = $this->context->currency;
        $total    = (float) $cart->getOrderTotal(true, Cart::BOTH);
        $mailVars = array();

        $this->module->validateOrder($cart->id, Configuration::get('PS_OS_PREPARATION'), $total, $this->module->displayName, null, $mailVars, (int) $currency->id, false, $customer->secure_key);

        Tools::redirect($this->_getCheckoutUri($cart));
    }

    private function _getCheckoutUri($cart)
    {
        $order            = new Order($this->module->currentOrder);
        $customer         = new Customer($cart->id_customer);
        $billing_address  = new Address($cart->id_address_invoice);
        $delivery_address = new Address($cart->id_address_delivery);

        // ====================== TRANSACTION ======================
        $amount = 0;
        $items  = $this->_calculateProducts($cart);
        foreach ($items as $item) {
            $amount += $item['price'] * $item['quantity'];
        }

        $params_transaction_details = array(
            'order_id' => (string) $order->id,
            'amount'   => $this->_determineValidNumber($amount),
            'items'    => $items,
        );

        // ====================== CUSTOMER ======================
        $params_customer = array(
            'first_name' => $customer->firstname,
            'last_name'  => $customer->lastname,
            'email'      => $customer->email,
            'phone'      => '',
        );

        // ====================== BILLING ADDRESS ======================
        $params_billing_address = array(
            'first_name'   => $billing_address->firstname,
            'last_name'    => $billing_address->lastname,
            'address'      => $billing_address->address1,
            'city'         => $billing_address->city,
            'postal_code'  => $billing_address->postcode,
            'phone'        => $this->_determineValidPhone($billing_address->phone, $billing_address->phone_mobile),
            'country_code' => 'IDN',
        );

        // ====================== SHIPPING ADDRESS ======================
        $params_shipping_address = array();
        if (!$cart->isVirtualCart()) {
            if ($cart->id_address_delivery != $cart->id_address_invoice) {
                $params_shipping_address = array(
                    'first_name'   => $delivery_address->firstname,
                    'last_name'    => $delivery_address->lastname,
                    'address'      => $delivery_address->address1,
                    'city'         => $delivery_address->city,
                    'postal_code'  => $delivery_address->postcode,
                    'phone'        => $this->_determineValidPhone($delivery_address->phone, $delivery_address->phone_mobile),
                    'country_code' => 'IDN',
                );
            } else {
                $params_shipping_address = $params_billing_address;
            }
        }

        try {

            // Setup Config
            Kredivo_Config::$is_production = $this->module->isProduction();

            $params = array(
                'server_key'          => Configuration::get('PS_KREDIVO_SERVER_KEY'),
                'payment_type'        => '30_days',
                'transaction_details' => $params_transaction_details,
                'customer_details'    => $params_customer,
                'billing_address'     => $params_billing_address,
                'shipping_address'    => $params_shipping_address,
                'push_uri'            => $this->module->getNotificationUri(),
                'order_status_uri'    => $this->module->getOrderStatusUri(),
                'back_to_store_uri'   => $this->module->getBaseUri(),
            );

            // echo json_encode($params);
            // echo "<pre>", print_r($params);
            // exit();

            Kredivo_Log::debug(array_merge([
                'production' => Kredivo_Config::$is_production,
                'endpoint'   => Kredivo_Config::get_api_endpoint(),
            ], $params));

            return Kredivo_Api::checkout_url($params);

        } catch (Exception $e) {}

        return 'index.php?controller=order&step=3';
    }

    public function _calculateProducts($cart)
    {
        $items    = array();
        $products = $cart->getProducts();

        if (sizeof($products) > 0) {
            foreach ($products as $item) {

                if ($item['quantity']) {
                    $category = new Category((int) $item['id_category_default'], (int) $this->module->getLanguageID());
                    $items[]  = array(
                        "id"       => $item['id_product'],
                        "name"     => $item['name'],
                        "price"    => $this->_determineValidNumber($item['price_wt']),
                        "quantity" => $this->_determineValidNumber($item['cart_quantity']),
                        "type"     => $category->name,
						"url"      => "http://".$_SERVER['HTTP_HOST'].'/product.php?id_product=' . $item['id_product'],
                    );
                }
            }
        }
		
        $shipping_cost = $cart->getTotalShippingCost();
        if ($shipping_cost != 0) {
            $items[] = array(
                "id"       => "shippingfee",
                "name"     => "Shipping Fee",
                "price"    => $this->_determineValidNumber($shipping_cost),
                "quantity" => 1,
            );
        }

        $discount = -1 * $cart->getOrderTotal(true, Cart::ONLY_DISCOUNTS);
        if (version_compare(Configuration::get('PS_VERSION_DB'), '1.5') == -1) {
            $discount *= -1;
        }
        if ($discount != 0) {
            $items[] = array(
                "id"       => "discount",
                "name"     => "Discount Voucher",
                "price"    => $this->_determineValidNumber($discount),
                "quantity" => 1,
            );
        }

        return $items;
    }

    public function _determineValidPhone($home_phone = '', $mobile_phone = '')
    {
        if (empty($home_phone) && !empty($mobile_phone)) {
            return $mobile_phone;
        } else if (!empty($home_phone) && empty($mobile_phone)) {
            return $home_phone;
        } else if (!empty($home_phone) && !empty($mobile_phone)) {
            return $mobile_phone;
        }

        return '';
    }

    public function _determineValidNumber($str)
    {
        try {
            return is_string($str) ? floatval($str) : $str;
        } catch (Exception $e) {}

        return $str;
    }
}
